"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpressFileServerContribution = void 0;
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const koa_mount_1 = tslib_1.__importDefault(require("koa-mount"));
const di_1 = require("@opensumi/di");
const ide_core_node_1 = require("@opensumi/ide-core-node");
const common_1 = require("../common");
let ExpressFileServerContribution = class ExpressFileServerContribution {
    getWhiteList() {
        return [
            // 插件市场安装目录
            this.appConfig.marketplace.extensionDir,
            ...(this.appConfig.staticAllowPath || []),
        ];
    }
    initialize(app) {
        app.use((0, koa_mount_1.default)('/assets', async (ctx) => {
            const uriPath = decodeURI(ctx.path.replace(/^\/assets/, ''));
            if (!uriPath) {
                ctx.status = 404;
                return;
            }
            const filePath = ide_core_node_1.URI.parse(`file://${uriPath}`).codeUri.fsPath;
            const whitelist = this.getWhiteList();
            const contentType = common_1.ALLOW_MIME[path_1.default.extname(filePath).slice(1)];
            if (
            /**
             * 地址在白名单内
             * Windows 下 C:\\Path\\to\\file 会被转换成 c:\\Path\\to\\file
             */
            whitelist.some((whitelistPath) => whitelistPath && filePath.toLowerCase().startsWith(whitelistPath.toLowerCase())) &&
                // 在允许的 contentType
                contentType) {
                ctx.set('Content-Type', contentType);
                if (this.appConfig.staticAllowOrigin) {
                    ctx.set('Access-Control-Allow-Origin', this.appConfig.staticAllowOrigin);
                }
                ctx.body = fs_1.default.createReadStream(filePath);
            }
            else {
                ctx.status = 403;
            }
        }));
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_node_1.AppConfig),
    tslib_1.__metadata("design:type", Object)
], ExpressFileServerContribution.prototype, "appConfig", void 0);
ExpressFileServerContribution = tslib_1.__decorate([
    (0, ide_core_node_1.Domain)(ide_core_node_1.ServerAppContribution)
], ExpressFileServerContribution);
exports.ExpressFileServerContribution = ExpressFileServerContribution;
//# sourceMappingURL=express-file-server.contribution.js.map